#<profile4xi profile = "20" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "pipe4xi.py">
#<abstract4xi abstract = "provides the function Pipe4Xi"/>
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>

# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Pipe4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import errno
import io
import itertools
import os
import pickle
import select
import socket
import struct
import sys
import time
import tempfile
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
from multiprocessing.connection import Pipe
#"
#/>
          
# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from arbitrary_address4xi import arbitrary_address4xi
from constant_system4xi import get_windows_system4xi
from constant_connection4xi import ConstantConnection4Xi 
from constant_family4xi import get_af_inet4xi
from constant_family4xi import get_af_unix4xi
from constant_family4xi import get_af_pipe4xi
from connection4xi import Connection4Xi
from examine4xi import examine4xi
from pipeconnection4xi import PipeConnection4Xi
from print4xi import print4xi
from time4xi import time4xi
#"
#/>


# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>
if sys.platform != get_windows_system4xi(): # dragonized
    # ---------------------------------------------------------------------------------------------    
    # Linux ...
    # ---------------------------------------------------------------------------------------------     
  

    # -------------------------------------------------------------------------------------------------
    #<function4xi function = "Pipe4Xi" usage = "
    # 
    # basemanager4xi.py:
    #     reader2xi, writer2xi = Pipe4Xi(duplex=False)
    # gun4xi.py:
    #     pipe2zero_consumer_channel, pipe2zero_publisher_channel = Pipe4Xi()
    # noun4xi.py:
    #     pipe2zero_consumer_channel, pipe2zero_publisher_channel = Pipe4Xi()
    # object4xi.py:
    #     pipe2one_consumer_channel, pipe2one_publisher_channel = Pipe4Xi()# simplequeue4xi.py:
    #     self._reader, self._writer = Pipe4Xi(duplex=False)
    # queue4xi.py:
    #     self._reader, self._writer = Pipe4Xi(duplex=False)  
    # simplequeue4xi.py:
    #     self._reader, self._writer = Pipe4Xi(duplex=False)
    # syncmanager4xi.py:
    #     reader2xi, writer2xi = Pipe4Xi(duplex=False)
    # verb4xi.py:
    #     pipe2zero_consumer_channel, pipe2zero_publisher_channel = Pipe4Xi()
    # zap2actor4xi.py:
    #     pipe2zero_consumer_channel, pipe2zero_publisher_channel = Pipe4Xi()
    # zap2noun4xi.py:
    #     pipe2zero_consumer_channel, pipe2zero_publisher_channel = Pipe4Xi()
    # zap2verb4xi.py:
    #     pipe2zero_consumer_channel, pipe2zero_publisher_channel = Pipe4Xi()
    # __init__.py:
    #     return Pipe4Xi(duplex)    
    #
    #"> 
    #<note4xi note = "NOT Windows version"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------
    def Pipe4Xi(duplex=True):
        '''
        Pipe4Xi function returns pair of connection objects at either end of a pipe. 
        '''
        # -----------------------------------------------------------------------------------------    
        # Linux ...
        # -----------------------------------------------------------------------------------------     
        class2print = "Pipe4Xi"
        method2print = "Pipe4Xi"
        
        message2print = "{started" + "; Pipe4Xi-started: " + str(Pipe4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function Pipe4Xi at 0x000000000354D7B8>
        
        message2print = "{passed" + "; duplex-passed-to-Pipe4Xi: " + str(duplex) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # False
        
        if duplex:
            # ---------------------------------------------------------------------------------    
            # Step 1/3 - Linux duplex
            # ---------------------------------------------------------------------------------           
            s1_2xi, s2_2xi = socket.socketpair()
            
            message2print = "{(Step 1/3a) got" + "; s1-2xi-got-by-Pipe4Xi: " + str(s1_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                            
            message2print = "{(Step 1/3b) got" + "; s2-2xi-got-by-Pipe4Xi: " + str(s2_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            # ---------------------------------------------------------------------------------    
            # Step 2/3 - Linux duplex
            # ---------------------------------------------------------------------------------         
            s1_2xi.setblocking(True)
            message2print = "{(Step 2/3a) performed" + "; setblocking-one2xi-performed-by-Pipe4Xi: " + str(s1_2xi.setblocking)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #  
            
            s2_2xi.setblocking(True)
            message2print = "{(Step 2/3b) performed" + "; setblocking-two2xi-performed-by-Pipe4Xi: " + str(s2_2xi.setblocking)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #             
                           
            # ---------------------------------------------------------------------------------    
            # Step 3/3 - Linux duplex
            # ---------------------------------------------------------------------------------         
            c1_2xi = Connection4Xi(s1_2xi.detach())
            
            message2print = "{(Step 3/3a) got" + "; c1-2xi-one2xi-set-by-Pipe4Xi: " + str(c1_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #         
               
            c2_2xi = Connection4Xi(s2_2xi.detach())
            
            message2print = "{(Step 3/3b) got" + "; c2-2xi-two2xi-set-by-Pipe4Xi: " + str(c2_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                


        else:
            # ---------------------------------------------------------------------------------    
            # Step 1/3 - Linux non-duplex 
            # ---------------------------------------------------------------------------------           
                      
            fd1_2xi, fd2_2xi = os.pipe()
            
            message2print = "{(Step 1/3a) got" + "; fd1-2xi-got-by-Pipe4Xi: " + str(fd1_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
            message2print = "{(Step 1/3b) got" + "; fd2-2xi-got-by-Pipe4Xi: " + str(fd2_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                    

            # ---------------------------------------------------------------------------------    
            # Step 2/3 - Linux non-duplex 
            # ---------------------------------------------------------------------------------         
            message2print = "{(Step 2/3a) passing" + "; fd1-2xi-passing-to-Connection4Xi-by-Pipe4Xi: " + str(fd1_2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
        
            message2print = "{(Step 2/3b) passing" + "; writable-passing-to-Connection4Xi-by-Pipe4Xi: " + str(writable) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                            
            
            c1_2xi = Connection4Xi(fd1_2xi, writable=False)
             
            message2print = "{(Step 2/3c) got" + "; c1-2xi-got-by-Pipe4Xi: " + str(c1_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
         
            
            # ---------------------------------------------------------------------------------    
            # Step 3/3 - Linux non-duplex 
            # ---------------------------------------------------------------------------------         
            message2print = "{(Step 3/3a) passing" + "; fd2-2xi-passing-to-Connection4Xi-by-Pipe4Xi: " + str(fd2_2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
        
            message2print = "{(Step 3/3b) passing" + "; readable-passing-to-Connection4Xi-by-Pipe4Xi: " + str(readable) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                         
                
            c2_2xi = Connection4Xi(fd2_2xi, readable=False)
          
   
            message2print = "{(Step 3/3c) got" + "; c2-2xi-got-by-Pipe4Xi: " + str(c2_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        
  

        # -------------------------------------------------------------------------------------    
        # Linux
        # -------------------------------------------------------------------------------------         

        message2print = "{returning" + "; c1-2xi-returning-by-Pipe4Xi: " + str(c1_2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{returning" + "; c2-2xi-returning-by-Pipe4Xi: " + str(c2_2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
                                  

        return c1_2xi, c2_2xi
#</function4xi>
         
else:
    # -------------------------------------------------------------------------------------------------    
    # Windows version
    # -------------------------------------------------------------------------------------------------     
             
    
    # -------------------------------------------------------------------------------------------------
    #<function4xi function = "Pipe4Xi" usage = "
    # 
    # basemanager4xi.py:
    #     reader2xi, writer2xi = Pipe4Xi(duplex=False)
    #
    # queue4xi.py:
    #     self._reader, self._writer = Pipe4Xi(duplex=False)
    #
    # simplequeue4xi.py:
    #     self._reader, self._writer = Pipe4Xi(duplex=False)
    #
    # syncmanager4xi.py:
    #     reader2xi, writer2xi = Pipe4Xi(duplex=False)
    #
    # __init__.py:
    #     return Pipe4Xi(duplex)    
    #
    #"> 
    #<note4xi note = "Windows version"/>
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------
    def Pipe4Xi(duplex=True):
        '''
        Returns pair of connection objects at either end of a pipe
        '''
        # -----------------------------------------------------------------------------------------    
        # Windows version
        # -----------------------------------------------------------------------------------------     
        class2print = "Pipe4Xi" 
        method2print = "Pipe4Xi" 

        message2print = "{started" + "; Pipe4Xi-started: " + str(Pipe4Xi)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #HP = hello_pipe4xi.py
        #HP: <function Pipe4Xi at 0x0000000002C9FD08>
        #VB = test_verbkb4xi.py
        #VB: <function Pipe4Xi at 0x000000000456D510>
        
        message2print = "{passed" + "; duplex-passed-to-Pipe4Xi: " + str(duplex) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: duplex: True
        #    non-duplex: False
        #PX: non-duplex: False <-- non duplex for test_proxy_mp_xi4.py, requires close from client !
        #VB: True 
                       
        if examine4xi(class2print, method2print): 
            # -------------------------------------------------------------------------------------    
            # StepOriginal Windows version
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) passing" + "; duplex-passing-to-Pipe-by-Pipe4Xi: " + str(duplex) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
                    
                    
            message2print = "{(StepOriginal) returning" + "; Pipe-function-returning-by-Pipe4Xi: " + str(Pipe) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
        
        
            return Pipe(duplex)
        
   
            
        # -------------------------------------------------------------------------------------    
        # (Step 1/8) - Windows duplex
        # -------------------------------------------------------------------------------------         
        message2print = "{(Step 1/8) got" + "; duplex-got-by-Pipe4Xi: " + str(duplex)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # False     
        
        # -------------------------------------------------------------------------------------    
        # (Step 2/8) - Windows duplex
        # -------------------------------------------------------------------------------------         
        pipe2xi = get_af_pipe4xi()

        message2print = "{(Step 2/8a) got" + "; pipe2xi-got-by-Pipe4Xi: " + str(pipe2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # AF_PIPE         
                  
        message2print = "{(Step 2/8b) passing" + "; pipe2xi-passing-to-arbitrary-address4xi-by-Pipe4Xi: " + str(pipe2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # AF_PIPE
        
  
        wave2pipe = arbitrary_address4xi(pipe2xi)    
        message2print = "{(Step 2/8c) set" + "; wave2pipe-set-by-Pipe4Xi to: " + str(wave2pipe)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # \\.\pipe\pyc-3616-0-ufqw6o
                   
        # -------------------------------------------------------------------------------------    
        # (Step 3/8) - Windows duplex
        # -------------------------------------------------------------------------------------         
        cc2xi = ConstantConnection4Xi()
        
        message2print = "{(Step 3/8a) got" + "; cc2xi-got-by-Pipe4Xi: " + str(cc2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <constant_connection4xi.ConstantConnection4Xi object at 0x0000000003690B70>
        
        buffersize2xi = cc2xi.get_buffersize()
        
        message2print = "{(Step 3/8b) set" + "; buffersize2xi-set-by-Pipe4Xi to: " + str(buffersize2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # 8192
        
        # -------------------------------------------------------------------------------------    
        # (Step 4/8) - Windows
        # -------------------------------------------------------------------------------------         
        if duplex:
            # -------------------------------------------------------------------------------------    
            # (Step 4/8) - Windows duplex 
            # -------------------------------------------------------------------------------------               
            # The pipe is bi-directional; both server and client processes can read from and write to the pipe. 
            # This mode gives the server the equivalent of GENERIC_READ and GENERIC_WRITE access to the pipe. 
            # The client can specify GENERIC_READ or GENERIC_WRITE, or both, when it connects to the pipe using the CreateFile function.
                        

            openmode2xi = _winapi.PIPE_ACCESS_DUPLEX
              
            message2print = "{(Step 4/8a) set" + "; openmode2xi-set-by-Pipe4Xi to: " + str(openmode2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #HP: 1

            access2xi = _winapi.GENERIC_READ | _winapi.GENERIC_WRITE 

            message2print = "{(Step 4/8b) set" + "; access2xi-set-by-Pipe4Xi to: " + str(access2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                             
            #HP: 1073741824
   
            obsize2xi, ibsize2xi = buffersize2xi, buffersize2xi        #dragonized BUFSIZE, BUFSIZE 
            

            message2print = "{(Step 4/8c) set" + "; obsize2xi-set-by-Pipe4Xi to: " + str(obsize2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            #HP: 8192  
            

            message2print = "{(Step 4/8d) set" + "; ibsize2xi-set-by-Pipe4Xi to: " + str(ibsize2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            #HP: 8192
        else:
         
            # -------------------------------------------------------------------------------------    
            # (Step 4/8) - Windows non-duplex 
            #
            # Queue4xi:
            #     __init__: 
            #         self._reader, self._writer = Pipe4Xi(duplex=False)
            # hello_pipe4xi.py:    
            #     wave2read, wave2write = Pipe4Xi(duplex=False)
            # -------------------------------------------------------------------------------------               
            openmode2xi = _winapi.PIPE_ACCESS_INBOUND
            
            # The flow of data in the pipe goes FROM CLIENT TO SERVER .i.e.  
            # reader = Server, writer = client
            # This mode gives the server the equivalent of GENERIC_READ access to the pipe.


            
            message2print = "{(Step 4/8f) set" + "; openmode2xi-set-by-Pipe4Xi to: " + str(openmode2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 1                          

   
            # -------------------------------------------------------------------------------------    
            # Windows NOT DUPLEX 
            # -------------------------------------------------------------------------------------     
            #
            # The client must specify GENERIC_WRITE access when connecting to the pipe.
            # If the client must read pipe settings by calling the GetNamedPipeInfo or GetNamedPipeHandleState functions, 
            # the client must specify GENERIC_WRITE and FILE_READ_ATTRIBUTES access when connecting to the pipe.
                            
            access2xi = _winapi.GENERIC_WRITE
           
            message2print = "{(Step 4/8g) set" + "; access2xi-set-by-Pipe4Xi to: " + str(access2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 1073741824
   
            # -------------------------------------------------------------------------------------    
            # Windows NOT DUPLEX 
            # -------------------------------------------------------------------------------------     
                
            obsize2xi, ibsize2xi = 0, buffersize2xi                    #dragonized BUFSIZE
            
              
            message2print = "{(Step 4/8h) set" + "; obsize2xi-set-by-Pipe4Xi to: " + str(obsize2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 0

            
               
            message2print = "{(Step 4/8i) set" + "; ibsize2xi-set-by-Pipe4Xi to: " + str(ibsize2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 8192

      
        # --------------------------------------------------------------------------------------------    
        # Step 5/8 create named Pipe
        # --------------------------------------------------------------------------------------------         
        try:
            h1_2xi = _winapi.CreateNamedPipe(
                wave2pipe, openmode2xi | _winapi.FILE_FLAG_OVERLAPPED |
            _winapi.FILE_FLAG_FIRST_PIPE_INSTANCE,
            _winapi.PIPE_TYPE_MESSAGE | _winapi.PIPE_READMODE_MESSAGE |
            _winapi.PIPE_WAIT,
            1, obsize2xi, ibsize2xi, _winapi.NMPWAIT_WAIT_FOREVER, _winapi.NULL
            )
        
             
            message2print = "{(Step 5/8a) set" + "; surfboard-one2xi-set-by-Pipe4Xi to: " + str(h1_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 332
            
        except:
           
            message2print = "{(Step Step 5/8z) hit" + "; exception-hit-by-Pipe4Xi: " + " *** (Step Step 5/8z) exception with _winapi.CreateNamedPipe *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                    
        # -------------------------------------------------------------------------------------    
        # h2_2xi: create File to the client end of the pipe 
        # -------------------------------------------------------------------------------------         
        try:        
         
            message2print = "{(Step 5/8c1) passing" + "; wave2pipe-passing-to-winapi-CreateFile-by-Pipe4Xi: " + str(wave2pipe) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # \\.\pipe\pyc-3616-0-ufqw6o
                       
            message2print = "{(Step 5/8c2) passing" + "; access2xi-passing-to-winapi-CreateFile-by-Pipe4Xi: " + str(access2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 1073741824
            
            message2print = "{(Step 5/8c3) passing" + "; 0-passing-to-winapi-CreateFile-by-Pipe4Xi: " + str(0) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 0
            
            message2print = "{(Step 5/8c4) passing" + "; winapi-NULL-passing-to-winapi-CreateFile-by-Pipe4Xi: " + str(_winapi.NULL) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 0
            
            message2print = "{(Step 5/8c5) passing" + "; winapi-OPEN-EXISTING-passing-to-winapi-CreateFile-by-Pipe4Xi: " + str(_winapi.OPEN_EXISTING) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 3;
            
            message2print = "{(Step 5/8c6) passing" + "; 0-passing-to-winapi-CreateFile-by-Pipe4Xi: " + str(0) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 0;
            
            message2print = "{(Step 5/8c7) passing" + "; winapi-NULL-passing-to-winapi-CreateFile-by-Pipe4Xi: " + str(_winapi.NULL) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 0
        
        
            h2_2xi = _winapi.CreateFile(wave2pipe, access2xi, 0, _winapi.NULL, _winapi.OPEN_EXISTING, _winapi.FILE_FLAG_OVERLAPPED, _winapi.NULL)
            
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
                 
            message2print = "{(Step 5/8c8) set" + "; surfboard-two2xi-set-by-Pipe4Xi to: " + str(h2_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 336
        except:
           
            message2print = "{(Step Step 5/8z) hit" + "; exception-hit-by-Pipe4Xi: " + " *** (Step Step 5/8z) exception with _winapi.CreateFile *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
        # -------------------------------------------------------------------------------------    
        # set State for the created file
        # -------------------------------------------------------------------------------------               
        # h2_2xi: the handle to the named pipe instance. 
        #        This parameter can be a handle to the server end of the pipe, as returned by the CreateNamedPipe function, or 
        #        to the client end of the pipe, as returned by the CreateFile function. 
        #        The handle must have GENERIC_WRITE access to the named pipe for a write-only or read/write pipe, or 
        #        it must have GENERIC_READ and FILE_WRITE_ATTRIBUTES access for a read-only pipe. 


        try:
            message2print = "{(Step 5/8e1) passing" + "; surfboard-two2xi-passing-to-winapi-SetNamedPipeHandleState-by-Pipe4Xi: " + str(h2_2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 336
                    
            message2print = "{(Step 5/8e2) passing" + "; winapi-PIPE-READMODE-MESSAGE-passing-to-winapi-SetNamedPipeHandleState-by-Pipe4Xi: " + str(_winapi.PIPE_READMODE_MESSAGE) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 2
            
            message2print = "{(Step 5/8e3) passing" + "; None-passing-to-winapi-SetNamedPipeHandleState-by-Pipe4Xi: " + str(None) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # None
            
            message2print = "{(Step 5/8e4) passing" + "; None-passing-to-winapi-SetNamedPipeHandleState-by-Pipe4Xi: " + str(None) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # None
                                                                  
            _winapi.SetNamedPipeHandleState(
                h2_2xi, _winapi.PIPE_READMODE_MESSAGE, None, None
                )
   
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            message2print = "{(Step 5/8e5) performed" + "; winapi-SetNamedPipeHandleState-performed-by-Pipe4Xi: " + str(_winapi.SetNamedPipeHandleState) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <built-in function SetNamedPipeHandleState>

        except:
            message2print = "{(Step Step 5/8e) hit" + "; exception-hit-by-Pipe4Xi: " + " *** (Step Step 5/8d) exception calling _winapi.SetNamedPipeHandleState *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        # -------------------------------------------------------------------------------------    
        # (Step 6/8) connect to named Pipe
        # -------------------------------------------------------------------------------------         
        overlapped2xi = None
        try:
            overlapped2xi = _winapi.ConnectNamedPipe(h1_2xi, overlapped=True)
            message2print = "{(Step 6/8a) got" + "; overlapped2xi-got-by-Pipe4Xi: " + str(overlapped2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
            # <_winapi.Overlapped object at 0x00000000036DD4F0>
            
            _, err2xi = overlapped2xi.GetOverlappedResult(True)
            
            message2print = "{(Step 6/8b) got" + "; err2xi-got-by-Pipe4Xi: " + str(err2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # 0
        
        except:
            message2print = "{(Step 6/8z) failed" + "; overlapped2xi-failed-by-Pipe4Xi: " + str(overlapped2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # 

        # -------------------------------------------------------------------------------------    
        # (Step 7/8)
        # -------------------------------------------------------------------------------------         
        message2print = "{(Step 7/8a1) passing" + "; surfboard-one2xi-passing-to-PipeConnection-by-Pipe4Xi: " + str(h1_2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # 332
                
        message2print = "{(Step 7/8a2) passing" + "; duplex-passing-to-PipeConnection-by-Pipe4Xi: " + str(duplex) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                 
        # False
        
        # -------------------------------------------------------------------------------------    
        # PipeConnection4Xi with handle 1
        # -------------------------------------------------------------------------------------          
        message2print = "{(Step 7/8b0) calling" + "; PipeConnection4Xi-calling-by-Pipe4Xi: " + str(PipeConnection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <class 'pipeconnection4xi.PipeConnection4Xi'>
        try:
            c1_2xi = PipeConnection4Xi(h1_2xi, writable=duplex)
            
           
            message2print = "{(Step 7/8b1) performed" + "; PipeConnection4Xi-performed-by-Pipe4Xi: " + str(PipeConnection4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <class 'pipeconnection4xi.PipeConnection4Xi'>
            
            message2print = "{(Step 7/8b2) got" + "; connection-one2xi-got-by-Pipe4Xi: " + str(c1_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003690A90>
                           
        except:
            # -------------------------------------------------------------------------------------    
            # 
            # -------------------------------------------------------------------------------------     
            message2print = "{(Step 7/8z) hit" + "; exception-hit-by-Pipe4Xi: " + " *** (Step 7/8c) failed with PipeConnection4Xi *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        
        # -------------------------------------------------------------------------------------    
        # PipeConnection4Xi with handle 2
        # -------------------------------------------------------------------------------------          
        message2print = "{(Step 7/8d1) passing" + "; surfboard-two2xi-passing-to-PipeConnection4Xi-by-Pipe4Xi: " + str(h2_2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # 336
        
        message2print = "{(Step 7/8d2) passing" + "; duplex-passing-to-PipeConnection4Xi-by-Pipe4Xi: " + str(duplex) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        # False
        
        try:
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------     
            
            c2_2xi = PipeConnection4Xi(h2_2xi, readable=duplex)

            message2print = "{(Step 7/8e1) performed" + "; PipeConnection4Xi-performed-by-Pipe4Xi: " + str(PipeConnection4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
            # <class 'pipeconnection4xi.PipeConnection4Xi'>
                    
            message2print = "{(Step 7/8e2) got" + "; connection-two2xi-got-by-Pipe4Xi: " + str(c2_2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003690CF8>
            
        except:
            message2print = "{(Step 7/8z) hit" + "; exception-hit-by-Pipe4Xi: " + " *** (Step 7/8z) failed with PipeConnection4Xi *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        # -------------------------------------------------------------------------------------    
        # Step 8/8 
        # -------------------------------------------------------------------------------------         
        message2print = "{(Step 8/8a) returning" + "; connection-one2xi-returning-by-Pipe4Xi: " + str(c1_2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003690A90>
        
        message2print = "{(Step 8/8b) returning" + "; connection-two2xi-returning-by-Pipe4Xi: " + str(c2_2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003690CF8>
        
        return c1_2xi, c2_2xi
#</function4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#
#<record4xi record = "
#   test_hello_pipe4xi.txt; both duplex and non-duplex modes
#   sense-hello-pipe4xi.xml
#   reason-hello-pipe4xi.xml
#   ace-hello-pipe4xi.xml
#
#   test_pipe1_mp_xi4.txt; uses Queue4Xi which calls Pipe4Xi with duplex=False <-- to be solved 
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#
#   test_proxy_mp_xi4.py.txt
#   sense-proxy-mp-test4xi.xml
#   reason-proxy-mp-test4xi.xml
#   ace-proxy-mp-test4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of pipe4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>



